<?php

declare(strict_types = 1);

namespace Drupal\commerce_otpsp\PluginForm;

use Cheppers\OtpspClient\OtpSimplePayClient;
use Drupal\commerce_otpsp\PaymentRequestBuilderInterface;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class SimplePayRedirectOffSitePaymentForm extends PaymentOffsiteForm implements ContainerInjectionInterface {

  /**
   * @var \Cheppers\OtpspClient\OtpSimplePayClient
   */
  protected $client;

  /**
   * @var \Drupal\commerce_otpsp\PaymentRequestBuilderInterface
   */
  protected $paymentRequestBuilder;

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('commerce_otpsp.client'),
      $container->get('commerce_otpsp.payment_request_builder')
    );
  }

  public function __construct(
    OtpSimplePayClient $client,
    PaymentRequestBuilderInterface $paymentRequestBuilder
  ) {
    $this->client = $client;
    $this->paymentRequestBuilder = $paymentRequestBuilder;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->getEntity();
    $payment->setRemoteState('INIT');
    $payment->save();

    $paymentRequest = $this->paymentRequestBuilder->getPaymentRequest($payment);

    $paymentResponse = $this->client->startPayment($paymentRequest);
    if ($paymentResponse->errorCodes !== []) {
      throw new PaymentGatewayException('Payment init failed', $paymentResponse->errorCodes[0]);
    }

    $payment->setRemoteId((string) $paymentResponse->transactionId)->save();

    return $this->buildRedirectForm(
      $form,
      $form_state,
      $paymentResponse->paymentUrl,
      $paymentRequest->jsonSerialize(),
      static::REDIRECT_POST
    );
  }

}
